@@ -36,7 +36,10 @@ from .queries import (CATALOG_QUERY,
                       FEED_QUERY,
                       PREREQUISITES_QUERY,
                       OFFERS_QUERY,
-                      STORE_QUERY)
+                      STORE_QUERY,
+                      ADDONS_QUERY,
+                      MEDIA_QUERY,
+                      PRODUCT_REVIEWS_QUERY)
 
 
 class OfferData(NamedTuple):
@@ -61,27 +64,22 @@ class EpicGamesStoreAPI:
         self.country = country
 
     def get_product_mapping(self) -> dict:
-        """
-        Returns product mapping in {namespace: slug} format
-        """
+        """Returns product mapping in {namespace: slug} format."""
         return self._make_api_query(
             '/content/productmapping', method='GET'
         )
 
     def get_product(self, slug: str) -> dict:
-        """
-        Returns a product's data by slug
+        """Returns a product's data by slug.
 
-        :param slug: Product's slug
+        :param slug: Product's slug.
         """
         return self._make_api_query(
             f'/content/products/{slug}', method='GET', use_locale=True
         )
 
     def get_store(self) -> dict:
-        """
-        Returns a JSON data about store page
-        """
+        """Returns a JSON data about store page."""
         return self._make_api_query(
             '/content/store', method='GET', use_locale=True
         )
@@ -105,9 +103,7 @@ class EpicGamesStoreAPI:
         )['result']
 
     def get_epic_store_status(self) -> dict:
-        """
-        Returns an Epic Games Store server status
-        """
+        """Returns an Epic Games Store server status."""
         return self._session.get(
             'https://status.epicgames.com/api/v2/status.json'
         ).json()
@@ -115,16 +111,18 @@ class EpicGamesStoreAPI:
     def get_offers_data(
         self,
         *offers: OfferData,
-        should_calculate_tax: bool = False
+        should_calculate_tax: bool = False,
+        include_sub_items: bool = False
     ) -> dict:
-        """
-        Get offer(s) full data by offers' id and namespace
+        """Get offer(s) full data by offers' id and namespace.
 
-        :param offers: Offers you need to get data from
+        :param offers: Offers you need to get data from.
         :param should_calculate_tax: Should EGS API calculate tax for offers?
+        :param include_sub_items: Should EGS API include sub-items for offers?
         """
         return self._make_graphql_query(
             OFFERS_QUERY,
+            {},
             *[{
                 'productNamespace': offer.namespace,
                 'offerId': offer.offer_id,
@@ -132,13 +130,86 @@ class EpicGamesStoreAPI:
                     'offerId': offer.offer_id,
                     'quantity': 1
                 }],
-                'calculateTax': should_calculate_tax
+                'calculateTax': should_calculate_tax,
+                'includeSubItems': include_sub_items
             } for offer in offers]
         )
 
-    def fetch_prerequisites(self, *offers: OfferData) -> dict:
+    def fetch_media(self, media_ref_id: str) -> dict:
+        """Returns media-file (type of the file, its url and so on) by the
+        file's media ref ID.
+
+        :param media_ref_id: File's media ref ID.
         """
-        Fetches offer(s) prerequisites
+        return self._make_graphql_query(
+            MEDIA_QUERY,
+            mediaRefId=media_ref_id
+        )
+
+    def fetch_multiple_media_files(self, *media_ref_ids: str):
+        """Equivalent to `fetch_media` function, except this one can fetch
+        a few media files at the same moment (using only one request)."""
+        return self._make_graphql_query(
+            MEDIA_QUERY,
+            {},
+            *[{
+                'mediaRefId': media_ref_id
+            } for media_ref_id in media_ref_ids]
+        )
+
+    def get_addons_by_namespace(
+        self,
+        namespace: str,
+        categories: str = 'addons|digitalextras',
+        count: int = 250,
+        sort_by: str = 'releaseDate',
+        sort_dir: str = 'DESC'
+    ):
+        """Returns product's addons by product's namespace.
+
+        :param namespace: Product's namespace, can be obtained using the
+        :meth:`epicstore_api.api.EpicGamesStoreAPI.get_product` function.
+
+        :param categories: Addon's categories.
+        :param count: Count of addon's you want EGS to give you.
+        :param sort_by: By which key EGS should sort addons.
+        :param sort_dir: You can use only **ASC** or **DESC**:
+
+        - **ASC**: Sorts from higher ``sort_by`` parameter to lower;
+        - **DESC**: Sorts from lower ``sort_by`` parameter to higher.
+        """
+        sort_dir = sort_dir.upper()
+        if sort_dir not in ('ASC', 'DESC'):
+            raise ValueError(f'Parameter ``sort_dir`` have to be equals to'
+                             f' ASC or DESC, not to {sort_dir}')
+        return self._make_graphql_query(
+            ADDONS_QUERY,
+            namespace=namespace,
+            count=count,
+            categories=categories,
+            sortBy=sort_by,
+            sortDir=sort_dir
+        )
+
+    def get_product_reviews(self, product_sku: str) -> dict:
+        """Returns product's reviews by product's sku.
+
+        :param product_sku: SKU of the Product. Usually just slug of the
+        product with `EPIC_` prefix."""
+        try:
+            return self._make_graphql_query(
+                PRODUCT_REVIEWS_QUERY,
+                sku=product_sku
+            )
+        except EGSNotFound as exc:
+            exc.message = (
+                'There are no reviews for this product, '
+                'or the given sku ({}) is incorrect.'.format(product_sku)
+            )
+            raise
+
+    def fetch_prerequisites(self, *offers: OfferData) -> dict:
+        """Fetches offer(s) prerequisites
 
         :param offers: Offer(s) we need to get prerequisites from
         """
@@ -156,12 +227,11 @@ class EpicGamesStoreAPI:
         count: int = 10,
         category: str = ''
     ) -> dict:
-        """
-        Fetches Epic Games Store feed by given params
+        """Fetches Epic Games Store feed by given params.
 
-        :param offset: From which news (index) we need to start
-        :param count: Count of the news we need to fetch
-        :param category: News categories
+        :param offset: From which news (index) we need to start.
+        :param count: Count of the news we need to fetch.
+        :param category: News categories.
         """
         return self._make_graphql_query(
             FEED_QUERY,
@@ -172,8 +242,7 @@ class EpicGamesStoreAPI:
         )
 
     def fetch_catalog_tags(self, namespace: str = 'epic') -> dict:
-        """
-        Fetches tags for a products with namespace ``namespace``
+        """Fetches tags for a products with namespace ``namespace``
 
         :param namespace: Products' namespace (**epic** = all)
         """
@@ -183,10 +252,9 @@ class EpicGamesStoreAPI:
         )
 
     def fetch_promotions(self, namespace: str = 'epic') -> dict:
-        """
-        Fetches a global promotions
+        """Fetches a global promotions.
 
-        :param namespace: Products' namespace (**epic** = all)
+        :param namespace: Products' namespace (**epic** = all).
         """
         return self._make_graphql_query(
             PROMOTIONS_QUERY,
@@ -204,23 +272,22 @@ class EpicGamesStoreAPI:
         keywords: str = '',
         categories: List[EGSCategory] = None
     ) -> dict:
-        """
-        Fetches a catalog with given parameters
+        """Fetches a catalog with given parameters
 
-        :param count: Count of  products you need to fetch
-        :param product_type: Product type(s) you need to get from EGS
-        :param namespace: Products namespace (epic = all namespaces)
-        :param sort_by: Parameter which EGS will use to sort products
+        :param count: Count of  products you need to fetch.
+        :param product_type: Product type(s) you need to get from EGS.
+        :param namespace: Products namespace (epic = all namespaces).
+        :param sort_by: Parameter which EGS will use to sort products.
         :param sort_dir: You can use only **ASC** or **DESC**:
 
-        - **ASC**: Sorts from higher ``sort_by`` parameter to lower
-        - **DESC**: Sorts from lower ``sort_by`` parameter to higher
+        - **ASC**: Sorts from higher ``sort_by`` parameter to lower;
+        - **DESC**: Sorts from lower ``sort_by`` parameter to higher.
 
-        :param start: From which game EGS should start
-        :param keywords: Search keywords
-        :param categories: Categories you need to fetch
+        :param start: From which game EGS should start.
+        :param keywords: Search keywords.
+        :param categories: Categories you need to fetch.
         :rtype: dict
-        :raises: ValueError  if ``sort_by`` not equals to **ASC** or **DESC**
+        :raises: ValueError  if ``sort_by`` not equals to **ASC** or **DESC**.
         """
         sort_dir = sort_dir.upper()
         if sort_dir not in ('ASC', 'DESC'):
@@ -258,22 +325,21 @@ class EpicGamesStoreAPI:
         categories: List[EGSCategory] = None,
         with_price: bool = True
     ) -> dict:
-        """
-        Fetches a store games with given parameters
+        """Fetches a store games with given parameters
 
-        :param count: Count of  products you need to fetch
-        :param product_type: Product type(s) you need to get from EGS
-        :param allow_countries: Products in the country. Default to 'US'
-        :param namespace: Products namespace ('' = all namespaces)
+        :param count: Count of  products you need to fetch.
+        :param product_type: Product type(s) you need to get from EGS.
+        :param allow_countries: Products in the country. Default to 'US'.
+        :param namespace: Products namespace ('' = all namespaces).
         :param sort_by: Parameter which EGS will use to sort products:
 
-        - **releaseDate**:  Sorts by release date
-        - **title**: Sorts by game title, alphabetical
+        - **releaseDate**:  Sorts by release date;
+        - **title**: Sorts by game title, alphabetical.
 
         :param sort_dir: You can use only **ASC** or **DESC**:
 
-        - **ASC**: Sorts from higher ``sort_by`` parameter to lower
-        - **DESC**: Sorts from lower ``sort_by`` parameter to higher
+        - **ASC**: Sorts from higher ``sort_by`` parameter to lower;
+        - **DESC**: Sorts from lower ``sort_by`` parameter to higher.
 
         :param release_date: Available when ``sort_by`` is 'releaseDate'.
 
@@ -281,12 +347,12 @@ class EpicGamesStoreAPI:
         - Example: '[2019-09-16T14:02:36.304Z, 2019-09-26T14:02:36.304Z]'
         - Leaving ``startDate`` or ``endDate`` blank will not limit start/end date.
 
-        :param start: From which game EGS should start
-        :param keywords: Search keywords
-        :param categories: Categories you need to fetch
-        :param with_price: To fetch price or not
+        :param start: From which game EGS should start.
+        :param keywords: Search keywords.
+        :param categories: Categories you need to fetch.
+        :param with_price: To fetch price or not.
         :rtype: dict
-        :raises: ValueError  if ``sort_by`` not equals to **ASC** or **DESC**
+        :raises: ValueError  if ``sort_by`` not equals to **ASC** or **DESC**.
         """
         sort_dir = sort_dir.upper()
         if sort_dir not in ('ASC', 'DESC'):
@@ -298,7 +364,7 @@ class EpicGamesStoreAPI:
             categories = EGSCategory.join_categories(*categories)
         if isinstance(product_type, EGSProductType):
             product_type = product_type.value
-        return self._make_graphql_query( #this type of fetch needs headers
+        return self._make_graphql_query(  # This type of fetch needs headers.
             STORE_QUERY,
             headers={'content-type': 'application/json;charset=UTF-8'},
             count=count,
@@ -385,10 +451,17 @@ class EpicGamesStoreAPI:
                 service_response = json.loads(
                     error['serviceResponse']
                 )
-                if service_response['errorCode'].endswith('not_found'):
-                    raise EGSNotFound(
-                        service_response['errorMessage'],
-                        service_response['numericErrorCode'],
-                        service_response
-                    )
+                if isinstance(service_response, dict):
+                    if service_response['errorCode'].endswith('not_found'):
+                        raise EGSNotFound(
+                            service_response['errorMessage'],
+                            service_response['numericErrorCode'],
+                            service_response
+                        )
+                elif isinstance(service_response, str):
+                    if service_response == 'not found':
+                        raise EGSNotFound(
+                            'The resource was not found, '
+                            'No more data provided by Epic Games Store.'
+                        )
                 # FIXME: Need to handle more errors than the code is handling now
